Titanium.fileSystemProxy = window.TitaniumFilesystem;

// we have to wrap the returned file object from Filesystem
// since he has optional parameters on some of his methods
// and the JNI bridge requires exact parameters or it will no-op
TitaniumFile = function(f) //Note: Not implemented on iPhone yet
{
	this.proxy = f;
};
TitaniumFile.prototype.isFile = function()
{
	return this.proxy.isFile();
};
TitaniumFile.prototype.isDirectory = function()
{
	return this.proxy.isDirectory();
};
TitaniumFile.prototype.isHidden = function()
{
	return this.proxy.isHidden();
};
TitaniumFile.prototype.isSymbolicLink = function()
{
	return this.proxy.isSymbolicLink();
};
TitaniumFile.prototype.isExecutable = function()
{
	return this.proxy.isExecutable();
};
TitaniumFile.prototype.isReadonly = function()
{
	return this.proxy.isReadonly();
};
TitaniumFile.prototype.isWriteable = function()
{
	return this.proxy.isWriteable();
};
TitaniumFile.prototype.resolve = function()
{
	return this.proxy.resolve();
};
TitaniumFile.prototype.read = function()
{
	return this.proxy.read();
};
TitaniumFile.prototype.write = function(data,append)
{
	append = typeof(append)=='undefined' ? false : append;
	return this.proxy.write(data,append);
};
TitaniumFile.prototype.readLine = function()
{
	return this.proxy.readLine();
};
TitaniumFile.prototype.copy = function(destination)
{
	return this.proxy.copy(destination);
};
TitaniumFile.prototype.move = function(destination)
{
	return this.proxy.move(destination);
};
TitaniumFile.prototype.rename = function(destination)
{
	return this.proxy.rename(destination);
};
TitaniumFile.prototype.createDirectory = function(recursive)
{
	recursive = typeof(recursive)=='undefined' ? false : recursive;
	return this.proxy.createDirectory(recursive);
};
TitaniumFile.prototype.deleteDirectory = function(recursive)
{
	recursive = typeof(recursive)=='undefined' ? false : recursive;
	return this.proxy.deleteDirectory(recursive);
};
TitaniumFile.prototype.deleteFile = function()
{
	return this.proxy.deleteFile();
};
TitaniumFile.prototype.getDirectoryListing = function()
{
	return this.proxy.getDirectoryListing();
};
TitaniumFile.prototype.getParent = function()
{
	return this.proxy.getParent();
};
TitaniumFile.prototype.exists = function()
{
	return this.proxy.exists();
};
TitaniumFile.prototype.createTimestamp = function()
{
	return this.proxy.createTimestamp();
};
TitaniumFile.prototype.modificationTimestamp = function()
{
	return this.proxy.modificationTimestamp();
};
TitaniumFile.prototype.name = function()
{
	return this.proxy.name();
};
TitaniumFile.prototype.extension = function()
{
	return this.proxy.extension();
};
TitaniumFile.prototype.size = function()
{
	return this.proxy.size();
};
TitaniumFile.prototype.nativePath = function()
{
	return this.proxy.nativePath();
};
TitaniumFile.prototype.spaceAvailable = function()
{
	return this.proxy.spaceAvailable();
};
TitaniumFile.prototype.setExecutable = function()
{
	return this.proxy.setExecutable();
};
TitaniumFile.prototype.setReadonly = function()
{
	return this.proxy.setReadonly();
};
TitaniumFile.prototype.setWriteable = function()
{
	return this.proxy.setWriteable();
};
TitaniumFile.prototype.toString = function()
{
	return String(this.proxy.toString());
};

TitaniumFile.createBlob = function(native) {
	var f = new TitaniumFile(native);
	function TitaniumBlob(f) {this.obj = f;};
	TitaniumBlob.prototype = f;

	var b = new TitaniumBlob(f);
	b.__defineGetter__("url", function() {
		return this.nativePath();
	});

	return b;
};

Titanium.Filesystem = {
	createTempFile : function() {
		return new TitaniumFile(Titanium.fileSystemProxy.createTempFile());
	},
	createTempDirectory : function() {
		return new TitaniumFile(Titanium.fileSystemProxy.createTempDirectory());
	},
	getFile : function(a,b,c,d,e,f,g) {
		var parts = [];
		if (arguments.length>0) parts.push(String(a));
		if (arguments.length>1) parts.push(String(b));
		if (arguments.length>2) parts.push(String(c));
		if (arguments.length>3) parts.push(String(d));
		if (arguments.length>4) parts.push(String(e));
		if (arguments.length>5) parts.push(String(f));
		if (arguments.length>6) parts.push(String(g));
		return new TitaniumFile(Titanium.fileSystemProxy.getFile(parts));
	},
	getFileStream : function(a,b,c,d,e,f,g) {
		var parts = [];
		if (arguments.length>0) parts.push(String(a));
		if (arguments.length>1) parts.push(String(b));
		if (arguments.length>2) parts.push(String(c));
		if (arguments.length>3) parts.push(String(d));
		if (arguments.length>4) parts.push(String(e));
		if (arguments.length>5) parts.push(String(f));
		if (arguments.length>6) parts.push(String(g));
		return TitaniumFile(Titanium.fileSystemProxy.getFileStream(parts));
	},
	getApplicationDirectory : function() {
		return new TitaniumFile(Titanium.fileSystemProxy.getApplicationDirectory());
	},
	getApplicationDataDirectory : function(private) {
		private = typeof(private)=='undefined' ? false : private;
		return new TitaniumFile(Titanium.fileSystemProxy.getApplicationDataDirectory(private));
	},
	getResourcesDirectory : function() {
		return new TitaniumFile(Titanium.fileSystemProxy.getResourcesDirectory());
	},
	getUserDirectory : function() {
		return new TitaniumFile(Titanium.fileSystemProxy.getUserDirectory());
	},
	getLineEnding : function() {
		return "\n";
	},
	getSeparator : function() {
		return ";";
	},
	getRootDirectories : function() {
		var results = Titanium.fileSystemProxy.getRootDirectories();
		if (results)
		{
			var files = [];
			for (var c=0;c<results.length;c++)
			{
				files[c].push(new TitaniumFile(results[c]));
			}
			return files;
		}
		return [];
	},
	asyncCopy : function() {
		return Titanium.fileSystemProxy.asyncCopy(arguments);
	},

	// mobile specific APIS
	isExternalStoragePresent: function() {
		return Titanium.fileSystemProxy.isExternalStoragePresent();
	}
};
