Titanium.uiProxy = window.TitaniumUI;

var MenuItem = function() {
	this.obj; // reference to Java object
	this._callback;

	this.isRoot = function() {
		return this.obj.isRoot();
	};
	this.isSeparator = function() {
		return this.obj.isSeparator(); // Valid data, but ignored by Android
	};
	this.isItem = function() {
		return this.obj.isItem();
	};
	this.isSubMenu = function() {
		return this.obj.isSubMenu();
	};
	this.isEnabled = function() {
		return this.obj.isEnabled();
	};
	this.addSeparator = function() {
		var m = new MenuItem;
		m.obj = this.obj.addSeparator();
		return m;
	};
	this.addItem = function(label, callback, icon) {
		var m = new MenuItem();
		this._callback = callback;
		m.obj = this.obj.addItem(label, registerCallback(this, this._callback), icon);
		return m;
	};
	this.addSubMenu = function(label, icon) {
		var m = new MenuItem();
		m.obj = this.obj.addSubMenu(label, icon);
		return m;
	};
	this.enable = function() {
		this.obj.enable();
	};
	this.disable = function() {
		this.obj.disable();
	};
	this.setLabel = function(label) {
		this.obj.setLabel(label);
	};
	this.getLabel = function() {
		return this.obj.getLabel();
	};
	this.setIcon = function(icon) {
		this.obj.setIcon(icon);
	};
	this.getIcon = function() {
		return this.obj.getIcon();
	};
	this.setCallback = function(f) {
		_callback = f;
		this.obj.setCallback(registerCallback(this, f));
	};
};

var OptionDialog = function(proxy) {
	this.proxy = proxy;

	this.setTitle = function(title) {
		this.proxy.setTitle(title);
	};
	this.setOptions = function(options) {
		var o = transformObjectValue(options, []);
		if (typeOf(o) !== 'array') {
			o = [ options ];
		}
		this.proxy.setOptions(o);
	};
	this.addEventListener = function(eventName, event) {
		if (eventName !== "click") {
			throw new Error("OptionDialog only handles click events. Use event name 'click'");
		}

		this.proxy.addListener('click', registerCallback(this, event));
	};
	this.show = function() {
		this.proxy.show();
	};
	// Noop, iPhone only methods
	this.setDestructive = function(id) {

	};
	this.setCancel = function(id) {

	}
};

var AlertDialog = function(proxy) {
	this.proxy = proxy;

	this.setTitle = function(title) {
		this.proxy.setTitle(title);
	};
	this.setMessage = function(msg) {
		alert("setting msg: " + msg);
		this.proxy.setMessage(msg);
	};
	this.setButtonNames = function(names) {
		var n = transformObjectValue(names, []);
		if (typeOf(n) !== 'array') {
			n = [ names ];
		}
		this.proxy.setButtons(n);
	};
	this.addEventListener = function(eventName, event) {
		if (eventName !== "click") {
			throw new Error("AlertDialog only handles click events. Use event name 'click'");
		}

		this.proxy.addListener('click', registerCallback(this, event));
	}
	this.show = function() {
		this.proxy.show();
	}
};

var ActivityIndicator = function(proxy) {
	this.proxy = proxy;

	this.setMessage = function(msg) {
		this.proxy.setMessage(msg);
	};
	this.setMin = function(n) {
		this.proxy.setMin(n);
	};
	this.setMax = function(n) {
		this.proxy.setMax(n);
	};
	this.setPos = function(n) {
		this.proxy.setPosition(n);
	};
	this.show = function() {
		this.proxy.show();
	};
	this.hide = function() {
		this.proxy.hide();
	};
	// See below for setLocation
};

var TitaniumNotifier = function(proxy) {
	this.proxy = proxy; // reference to Java object
	this._callback; //

	this.setTitle = function(title) {
		this.proxy.setTitle(title);
	};
	this.setMessage = function(message) {
		this.proxy.setMessage(message);
	};
	this.setIcon = function(iconUrl) {
		this.proxy.setIcon(iconUrl);
	};
	this.setDelay = function(delay) {
		this.proxy.setDelay(delay);
	};
	this.setCallback = function(callback) {
		this._callback = callback;
		this.proxy.setCallback(registerCallback(this, _callback));
	};
	this.show = function(animate, autohide) {

		this.proxy.show(transformObjectValue(animate, false),
				transformObjectValue(autohide, true));
	};
	this.hide = function(animate) {
		this.proxy.hide(transformObjectValue(animate, false));
	}
};

var UserWindow = function(proxy) {
	this.proxy = proxy; // reference to java object
	this._window; // the DOM window

	this.setWindowId = function(name) { //TODO: Is this to be exposed or not? -blain
		this.proxy.setWindowId(name);
	}
	this.setURL = function(url) {
		this.proxy.setUrl(url);
	};
	this.setTitle = function(title) {
		this.proxy.setTitle(title);
	};
	this.setTitleImage = function(imageUrl) {
		this.proxy.setTitleImage(imageUrl);
	};
	this.setFullscreen = function(fullscreen) {
		this.proxy.setFullscreen(fullscreen);
	};
	this.setType = function(type) {
		this.proxy.setType(type);
	}
	this.open = function(options) {
		this._window = this.proxy.open(); // Handle options later
		// append to windows
		return this._window;
	};
	this.close = function(options) {
		this.proxy.close();
		// Remove from windows
		this._window = null;
	};
	this.addEventListener = function(eventName, listener) {
		return this.proxy.addEventListener(eventName, registerCallback(this, listener));
	};
	this.removeEventListener = function(eventName, listenerId) {
		this.proxy.removeEventListener(eventName, listenerId);
	};
	// IPhone only methods
	this.setNavBarColor = function (color) {

	};
	this.setLeftNavButton = function(button) {

	};
	this.setRightNavButton = function(button) {

	};
	this.showNavBar = function(options) {

	};
	this.hideNavBar = function(options) {

	}
	this.setBarColor = function(options) {

	}
};

UserWindow.prototype.__defineGetter__("window", function() {
	return this._window;
});

Titanium.UI = {
	WINDOW_TABBED : 'tabbed',
	WINDOW_NORMAL : 'single',
	createWindow : function(options) {
		var w = new UserWindow(Titanium.uiProxy.createWindow());
		if (!isUndefined(options)) {
			var url = options['url'];
			var fullscreen = options['fullscreen'];
			var type = options['type'];
			var title = options['title'];
			var titleImage = options['titleImage'];

			if (!isUndefined(url)) {
				w.setURL(url);
			}
			if (!isUndefined(fullscreen)) {
				w.setFullscreen(fullscreen);
			}
			if (!isUndefined(type)) {
				w.setType(type);
			}
			if (!isUndefined(title)) {
				w.setTitle(title);
			}
			if (!isUndefined(titleImage)) {
				w.setTitleImage(titleImage);
			}
		}

		return w;
	},
	createMenu : function() {
		var m = new MenuItem;
		m.obj = Titanium.uiProxy.createMenu(); // Consider a hash of menus
		return m;
	},
	createTrayMenu : function() {
		//TODO implement UI.createTrayMenu
	},
	setMenu : function(m) {
		Titanium.uiProxy.setMenu(m.obj);
	},
	getMenu : function() {
		Titanium.uiProxy.getMenu(); //Not sure what this should/will do
	},
	setContextMenu : function() {
		//TODO implement UI.setContextMenu
	},
	getContextMenu : function() {
		//TODO implement UI.getContextMenu
	},
	setIcon : function() {
		//TODO implement UI.setIcon
	},
	windows : function() {
		//TODO implement UI.windows as value
	},
	createAlertDialog : function(options) {
		var dlg = new AlertDialog(Titanium.uiProxy.createAlertDialog());

		if (! isUndefined(options)) {
			title = options['title'];
			message = options['message'];
			buttonNames = options['buttonNames'];

			if (!isUndefined(title)) {
				dlg.setTitle(title);
			}
			if (!isUndefined(message)) {
				dlg.setMessage(message);
			}
			if (!isUndefined(buttonNames)) {
				dlg.setButtonNames(buttonNames);
			}
		}

		return dlg;
	},
	createOptionDialog : function(options) {
		var dlg = new OptionDialog(Titanium.uiProxy.createOptionDialog());

		if (! isUndefined(options)) {
			title = options['title'];
			optionValues = options['options'];

			if (!isUndefined(title)) {
				dlg.setTitle(title);
			}
			if (!isUndefined(buttonNames)) {
				dlg.setOptions(optionValues);
			}
		}

		return dlg;
	},
	createActivityIndicator : function(options) {
		var ind = new ActivityIndicator(Titanium.uiProxy.createProgressDialog());

		if (!isUndefined(options)) {
			var message = options['message'];
			var loc = options['location'];
			var type = options['type'];
			var minVal = options['min'];
			var maxVal = options['max'];
			var position = options['pos'];

			if (!isUndefined(message)) {
				ind.setMessage(message);
			}
			if (!isUndefined(loc)) {
				ind.setLocation(loc);
			}
			if (!isUndefined(type)) {
				ind.setType(type);
			}
			if (!isUndefined(minVal)) {
				ind.setMin(minVal);
			}
			if (!isUndefined(maxVal)) {
				ind.setMax(maxVal);
			}
			if (!isUndefined(position)) {
				ind.setPos(position);
			}
		}
		return ind;
	},

	// createNotification is below. It needs the property currentWindow
	// iPhone only methods
	createButton : function(options) {
		return null;
	},
	createToolbar : function(options) {
		return null;
	},
	setTabBadge : function(id) {
		// do nothing
	},
	setStatusBarColor : function(color) {
		// do nothing
	}
};
Titanium.UI.createAlert = Titanium.UI.createAlertDialog; //TODO remove

Titanium.UI._currentWindow = null;
Titanium.UI.__defineGetter__("currentWindow", function(){
	// Can't set this from the native side, so set on first access
	if (Titanium.UI._currentWindow == null) {
		Titanium.UI._currentWindow = new UserWindow(Titanium.uiProxy.getCurrentWindow());
	}
	return Titanium.UI._currentWindow;
});

Titanium.UI.ActivityIndicator = {
	STATUS_BAR : 0,
	DIALOG : 1,
	INDETERMINANT : 0,
	DETERMINANT : 1
};

Titanium.UI.Android = {

};

Titanium.UI.Android.SystemIcon = {
	ACTION : 'ti:Ti:default_icon',
	CAMERA : 'ti:Sys:ic_menu_camera',
	COMPOSE: 'ti:Sys:ic_menu_compose',
	BOOKMARKS : 'ti:Ti:default_icon',
	SEARCH : 'ti:Sys:ic_menu_search',
	ADD : 'ti:Sys:ic_menu_add',
	TRASH : 'ti:Sys:ic_menu_delete',
	ORGANIZE : 'ti:Sys:ic_menu_archive',
	REPLY : 'ti:Ti:default_icon',
	STOP : 'ti:Sys:ic_menu_stop',
	REFRESH : 'ti:Sys:ic_menu_refresh',
	PLAY : 'ti:Sys:ic_media_play',
	FAST_FORWARD : 'ti:Sys:ic_media_ff',
	PAUSE : 'ti:Sys:ic_media_pause',
	REWIND : 'ti:Sys:ic_media_rew',
	EDIT : 'ti:Sys:ic_menu_edit',
	CANCEL : 'ti:Sys:ic_menu_close_clear_cancel',
	SAVE : 'ti:Sys:ic_menu_save',
	DONE : 'ti:Sys:ic_menu_mark',
	// Android Only?
	BACK : 'ti:Sys:ic_menu_back',
	FORWARD : 'ti:Sys:ic_menu_forward',
	HELP : 'ti:Sys:ic_menu_help',
	HOME : 'ti:Sys:ic_menu_home',
	NEXT : 'ti:Sys:ic_media_next',
	PREFERENCES : 'ti:Sys:ic_menu_preferences',
	PREVIOUS : 'ti:Sys:ic_media_previous',
	REVERT : 'ti:Sys:ic_menu_revert',
	SEND : 'ti:Sys:ic_menu_send',
	SHARE : 'ti:Sys:ic_menu_share',
	VIEW : 'ti:Sys:ic_menu_view',
	ZOOM : 'ti:Sys:ic_menu_zoom'
};

ActivityIndicator.prototype.setLocation = function(loc) {
	if (! (loc == Titanium.UI.ActivityIndicator.STATUS_BAR ||
			loc == Titanium.UI.ActivityIndicator.DIALOG)) {
		throw new Error("Unsupported indicator location.");
	}
	this.proxy.setLocation(loc);
};
ActivityIndicator.prototype.setType = function(type) {
	if (!(type == Titanium.UI.ActivityIndicator.DETERMINANT ||
			type == Titanium.UI.ActivityIndicator.INDETERMINANT)) {
		throw new Error("Unsupported indicator type.");
	}
	this.proxy.setType(type);
};

Titanium.UI.createNotification = function(options)
{
	proxy = Titanium.uiProxy.createNotification();
	notifier = null;
	if (proxy != null) {
		notifier = new TitaniumNotifier(proxy);
		if (options != null) {
			var title = options['title'];
			var message = options['message'];
			var color = options['color'];
			var delay = options['delay'];
			var transparency = options['transparency'];

			if (!isUndefined(title)) {
				notifier.setTitle(title);
			}
			if (!isUndefined(message)) {
				notifier.setMessage(message);
			}
			if (!isUndefined(color)) {
				notifier.setColor(color);
			}
			if (!isUndefined(delay)) {
				notifier.setDelay(delay);
			}
			if (!isUndefined(transparency)) {
				notifier.setTransparency(transparency);
			}
		}
	} else {
		Titanium.API.warn("Unable to create proxy, returning null.");
	}
	return notifier;
};

Titanium.UI.iPhone = {
	BORDERED : -1,
	StatusBar : {
		OPAQUE_BLACK : -1
	},
	setStatusBarStyle : function(x) {

	},
	SystemButton : {
		PAUSE : -1,
		REWIND : -1,
		PLAY : -1,
		FIXED_SPACE : -1,
		FLEXIBLE_SPACE : -1,
	}
};

